@@ -6,7 +6,34 @@ use syn::{
     NestedMeta,
 };
 
-const ON_MONTH_IN_SECONDS: u64 = 2592_000;
+const ONE_MONTH_IN_SECONDS: u64 = 2592_000;
+const ONE_MINUTE_IN_SECONDS: u64 = 60;
+
+fn get_lit_int(lit: Option<&Lit>, default_value: u64) -> u64 {
+    match lit {
+        Some(exp_lit) => {
+            if let Lit::Int(exp_lit_int) = exp_lit {
+                exp_lit_int.base10_digits().parse::<u64>().unwrap()
+            } else {
+                default_value
+            }
+        }
+        None => default_value,
+    }
+}
+
+fn get_lit_str(lit: Option<&Lit>, default_value: String) -> String {
+    match lit {
+        Some(exp_lit) => {
+            if let Lit::Str(exp_lit_str) = exp_lit {
+                exp_lit_str.value()
+            } else {
+                default_value
+            }
+        }
+        None => default_value,
+    }
+}
 
 fn parse_invocation(attr: Vec<NestedMeta>, input: DeriveInput) -> TokenStream {
     let mut attr_into_iter = attr.into_iter();
@@ -41,7 +68,7 @@ fn parse_invocation(attr: Vec<NestedMeta>, input: DeriveInput) -> TokenStream {
     }
 
     let mut hashmap: HashMap<String, Lit> = HashMap::new();
-    for attr_iter in attr_into_iter.next() {
+    for attr_iter in attr_into_iter.into_iter() {
         if let NestedMeta::Meta(meta) = attr_iter {
             if let Meta::NameValue(namevalue) = meta {
                 let name = namevalue.path;
@@ -52,31 +79,20 @@ fn parse_invocation(attr: Vec<NestedMeta>, input: DeriveInput) -> TokenStream {
         }
     }
 
-    let exp = match hashmap.get("exp") {
-        Some(exp_lit) => {
-            if let Lit::Int(exp_lit_int) = exp_lit {
-                exp_lit_int.base10_digits().parse::<u64>().unwrap()
-            } else {
-                ON_MONTH_IN_SECONDS
-            }
-        }
-        None => ON_MONTH_IN_SECONDS,
-    };
+    let exp = get_lit_int(hashmap.get("exp"), ONE_MONTH_IN_SECONDS);
+    let leeway = get_lit_int(hashmap.get("leeway"), ONE_MINUTE_IN_SECONDS);
+    let cookie_key = get_lit_str(hashmap.get("cookie"), "".to_string());
+    let query_key = get_lit_str(hashmap.get("query"), "".to_string());
 
     // handle input
     let guard_type = &input.ident;
     let vis = &input.vis;
-    let fairing_name = format!("'{}' JwtFairing", "Test");
+    let fairing_name = format!("'{}' JwtFairing", &guard_type.to_string());
     let guard_claim = format_ident!("{}JwtClaim", &guard_type);
 
     let jwt = quote!(::jsonwebtoken);
     #[allow(non_snake_case)]
     let Result = quote!(::jsonwebtoken::errors::Result);
-    #[allow(non_snake_case)]
-    let Error = quote!(::jsonwebtoken::errors::Error);
-    #[allow(non_snake_case)]
-    let ErrorKind = quote!(::jsonwebtoken::errors::ErrorKind);
-
     #[allow(non_snake_case)]
     let Status = quote!(::rocket::http::Status);
     #[allow(non_snake_case)]
@@ -120,11 +136,14 @@ fn parse_invocation(attr: Vec<NestedMeta>, input: DeriveInput) -> TokenStream {
             }
 
             pub fn decode(token: String) -> #Result<#guard_claim> {
-                let result = #jwt::decode::<#guard_claim>(&token, &#jwt::DecodingKey::from_secret((#secrete_value).as_bytes()), &#jwt::Validation::default());
-                if let Ok(token_claim) = result {
-                    return Ok(token_claim.claims);
+                let mut validation = #jwt::Validation::default();
+                validation.leeway = #leeway;
+
+                let result = #jwt::decode::<#guard_claim>(&token, &#jwt::DecodingKey::from_secret((#secrete_value).as_bytes()), &validation);
+                match result {
+                    Ok(token_claim) => Ok(token_claim.claims),
+                    Err(err) => Err(err),
                 }
-                Err(#Error::from(#ErrorKind::InvalidToken))
             }
         }
 
@@ -132,12 +151,46 @@ fn parse_invocation(attr: Vec<NestedMeta>, input: DeriveInput) -> TokenStream {
             type Error = #response::status::Custom<String>;
 
             fn from_request(request: &'a #request::Request<'r>,) -> #request::Outcome<Self, #response::status::Custom<String>> {
-                if let Some(auth_header) = request.headers().get_one("Authorization") {
-                    let auth_str = auth_header.to_string();
+                let mut auth_str: Option<String> = None;
+                if (#cookie_key) != "" {
+                    auth_str = match request.cookies().get(#cookie_key) {
+                        None => None,
+                        Some(t) => Some(t.value().to_string()),
+                    };
+                } else if (#query_key) != "" {
+                    auth_str = match request.get_query_value::<String>(#query_key) {
+                        None => None,
+                        Some(t) => match t {
+                            Ok(r) => Some(r),
+                            Err(_) => None,
+                        }
+                    }
+                } else {
+                    auth_str = match auth_str {
+                        Some(auth_str) => Some(auth_str),
+                        None => match request.headers().get_one("Authorization") {
+                            Some(s) => Some(s.to_string()),
+                            None => None,
+                        }
+                    };
+                };
+
+                if let Some(auth_str) = auth_str {
                     if auth_str.starts_with("Bearer") {
                         let token = auth_str[6..auth_str.len()].trim();
-                        if let Ok(token_data) = #guard_type::decode(token.to_string()) {
-                            return #Outcome::Success(token_data.user);
+                        match #guard_type::decode(token.to_string()) {
+                            Ok(token_data) => {
+                                return #Outcome::Success(token_data.user);
+                            },
+                            Err(err) => {
+                                return #Outcome::Failure((
+                                    #Status::Unauthorized,
+                                    #response::status::Custom(
+                                        #Status::Unauthorized,
+                                        err.to_string(),
+                                    ),
+                                ));
+                            },
                         }
                     }
                 }
@@ -146,7 +199,7 @@ fn parse_invocation(attr: Vec<NestedMeta>, input: DeriveInput) -> TokenStream {
                     #Status::Unauthorized,
                     #response::status::Custom(
                         #Status::Unauthorized,
-                        String::from("401 Unauthorized"),
+                        String::from("EmptySignature"),
                     ),
                 ))
             }
@@ -198,7 +251,7 @@ fn parse_invocation(attr: Vec<NestedMeta>, input: DeriveInput) -> TokenStream {
 ///     token
 /// }
 ///
-/// #[get("/uer_id")]
+/// #[get("/user_id")]
 /// fn get_uer_id_from_jwt(user: UserClaim) -> String {
 ///     format!("user id is {}", user.id)
 /// }
@@ -210,7 +263,21 @@ fn parse_invocation(attr: Vec<NestedMeta>, input: DeriveInput) -> TokenStream {
 ///         .launch();
 /// }
 /// ```
+/// token default comes from request.header, if want get from cookie or query, user
 ///
+/// ```rust
+/// #[jwt("secret", cookie = "token")]
+/// pub struct UserClaim {
+///     id: String,
+/// }
+/// ```
+///
+/// /// ```rust
+/// #[jwt("secret", query = "token")]
+/// pub struct UserClaim {
+///     id: String,
+/// }
+/// ```
 #[proc_macro_attribute]
 pub fn jwt(attr: TokenStream, input: TokenStream) -> TokenStream {
     let input = parse_macro_input!(input as DeriveInput);
